@@ -13,25 +13,27 @@
 #'
 #' @export
 #' @aliases writexl
-#' @useDynLib writexl C_write_data_frame
-#' @param x data frame to write to disk
+#' @useDynLib writexl C_write_data_frame_list
+#' @param x data frame or named list of data frames that will be sheets in the xlsx
 #' @param path a file name to write to
 #' @param col_names write column names at the top of the file?
 #' @examples # Roundtrip example
 #' tmp <- write_xlsx(iris)
 #' readxl::read_xlsx(tmp)
 write_xlsx <- function(x, path = tempfile(fileext = ".xlsx"), col_names = TRUE){
-  stopifnot(is.data.frame(x))
+  if(is.data.frame(x))
+    x <- list(x)
+  if(!is.list(x) || !all(vapply(x, is.data.frame, logical(1))))
+    stop("Argument x must be a data frame or list of data frames")
+  x <- lapply(x, normalize_df)
   stopifnot(is.character(path) && length(path))
   path <- normalizePath(path, mustWork = FALSE)
-  df <- normalize_df(x)
-  headers <- if(isTRUE(col_names))
-    colnames(x)
-  .Call(C_write_data_frame, df, path, headers)
+  .Call(C_write_data_frame_list, x, path, col_names)
 }
 
 normalize_df <- function(df){
-  for(i in which(vapply(df, is.factor, logical(1)))){
+  # Types to coerce to strings
+  for(i in which(vapply(df, inherits, logical(1), c("Date", "factor", "hms")))){
     df[[i]] <- as.character(df[[i]])
   }
   for(i in which(vapply(df, inherits, logical(1), "POSIXlt"))){
